/***************************************************************************
                          kcanvas.cpp  -  description
                             -------------------
    begin                : Fri Jul 12 2002
    copyright            : (C) 2002 by Marco Krohn
    email                : marco.krohn@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <algorithm>
#include <cmath>

#include <kpixmap.h>
#include <qpainter.h>

#include "kcanvas.h"

#include "common/point.h"

using namespace std;





//----------------------------------------  class KCanvas  ----------------------------------------

KCanvas :: KCanvas( double w, double h )
{
  fWidth = w;
  fHeight = h;
  fMinSize = 0;
}



// virtual method
KCanvas :: ~KCanvas()
{
list<KCanvasItem*>::iterator it;
list<KCanvasView*>::iterator it2;

  // delete all canvas items
  for ( it=fCIList.begin(); it!=fCIList.end(); it++ )  {
    delete *it;
  }

  // delete all views
  for ( it2=fViewList.begin(); it2!=fViewList.end(); it2++ )  {
    delete *it2;
  }
}



void KCanvas :: addCanvasItem( KCanvasItem* ci )
{
  fCIList.push_back( ci );
}



void KCanvas :: addView( KCanvasView* view )
{
  fViewList.push_back( view );
}



const list<KCanvasItem*>* KCanvas :: getCanvasItemList() const
{
  return &fCIList;
}



double KCanvas :: getHeight() const
{
  return fHeight;
}



double KCanvas :: getWidth() const
{
  return fWidth;
}



bool KCanvas :: removeView( KCanvasView* view )
{
  list<KCanvasView*>::iterator it = find( fViewList.begin(), fViewList.end(), view );
  if ( it == fViewList.end() )  return false;
  delete *it;
  fViewList.erase( it );
  return true;
}





//----------------------------------------  private  ----------------------------------------

int KCanvas :: getMinSize() const
{
  return fMinSize;
}



void KCanvas :: setMinSize( int radius )
{
  if ( radius > fMinSize )  fMinSize = radius;
}





//----------------------------------------  class KCanvasItem  ----------------------------------------

KCanvasItem :: ~KCanvasItem()
{
}



void KCanvasItem :: hasChanged() const
{
  // FIXME
}



void KCanvasItem :: setMinSize( int radius )
{
  fCanvas->setMinSize( radius );
}





//----------------------------------------  protected  ----------------------------------------

KCanvasItem :: KCanvasItem( KCanvas* canvas )
{
  fCanvas = canvas;
}





//----------------------------------------  class KCanvasView  ----------------------------------------

// parent=0, name=0, w=0
KCanvasView :: KCanvasView( KCanvas* canvas, QWidget* parent, const char* name, WFlags w )
  : KScrollView( parent, name, w )
{
  fCanvas = canvas;
  fZoom = -1;
}



// virtual method
KCanvasView :: ~KCanvasView()
{
}



double KCanvasView :: getZoom() const
{
  return fZoom;
}



// smoothness=1
void KCanvasView :: fastZoomIn( double zoom, int x, int y, int smoothness )
{
int newx, newy, i;

  if ( validateZoom(zoom) == false )  return;
  if ( zoom <= fZoom )  return;                            // no zoom in

  double step = pow( zoom/fZoom, (double) 1/smoothness );
  for ( i=1; i<=smoothness; i++ )  {
    zoom *= step;
    newx = (int) ceil( (contentsX()+x) * zoom/fZoom - x );
    newy = (int) ceil( (contentsY()+y) * zoom/fZoom - y );

    fZoom = zoom;
    resizeContents( ceil(fCanvas->getWidth()*fZoom), ceil(fCanvas->getHeight()*fZoom) );  // fixme: , true
    setContentsPos( newx, newy, false );
    updateScrollBars();

    repaintAll();
  }

  emit zoomChanged( fZoom );
}



// smoothness=1
void KCanvasView :: fastZoomOut( double zoom, int x, int y, int smoothness )
{
int newx, newy, i;

  if ( validateZoom(zoom) == false )  return;
  if ( zoom >= fZoom )  return;                            // no zoom out

  double step = pow( zoom/fZoom, (double) 1/smoothness );
  for ( i=1; i<=smoothness; i++ )  {
    zoom *= step;

    // newx, newy is the point we want to fix
    newx = (int) ceil( (contentsX()+x) * zoom/fZoom - x );
    newy = (int) ceil( (contentsY()+y) * zoom/fZoom - y );

    fZoom = zoom;
    resizeContents( ceil(fCanvas->getWidth()*fZoom), ceil(fCanvas->getHeight()*fZoom) );  // fixme: , false
    setContentsPos( newx, newy, false );
    updateScrollBars();

    repaintAll();
  }

  emit zoomChanged( fZoom );
}



void KCanvasView :: repaintAll()
{
  viewport()->repaint( 0, 0, visibleWidth(), visibleHeight(), false );
}



void KCanvasView :: setZoom( double zoom )
{
  if ( validateZoom(zoom) == false )  return;
  if ( zoom == fZoom )  return;
  fZoom = zoom;

  resizeContents( ceil(fCanvas->getWidth()*fZoom), ceil(fCanvas->getHeight()*fZoom) );
  repaintAll();

  emit zoomChanged( fZoom );
}





//----------------------------------------  protected  ----------------------------------------

// virtual method
void KCanvasView :: contentsMouseMoveEvent( QMouseEvent *e )
{
  TPoint pos;
  pos.setX( (double) e->pos().x() / fZoom );
  pos.setY( (double) e->pos().y() / fZoom );
  emit mousePosChanged( pos );
}


#include <kcanvasitems.h> // fixme
#include <gamelogic/stobj.h>
#include <qpopupmenu.h>
#include <qcursor.h>
// virtual method
void KCanvasView :: contentsMousePressEvent( QMouseEvent *e )
{
  if ( e->button() != Qt::RightButton )  return;

  TPoint pos, dist;
  pos.setX( (double) e->pos().x() / fZoom );
  pos.setY( (double) e->pos().y() / fZoom );

  QPopupMenu* popup = new QPopupMenu( this, "menu" );

  list<KCanvasItem*>::const_iterator begin = fCanvas->getCanvasItemList()->begin();
  list<KCanvasItem*>::const_iterator it, end = fCanvas->getCanvasItemList()->end();
  for ( it=begin; it!=end; it++ )  {
    dist = (*it)->getBoundingBox().getCenter() - pos;
    if ( dist.getLength()*fZoom < 15 )  {
      KCIStellarObject* kstobj = dynamic_cast<KCIStellarObject*>( *it );
      popup->insertItem( kstobj->getStellarObject()->getName().c_str() );
    }
  }

  popup->setMouseTracking( true );
  popup->exec( QCursor::pos() );
  delete popup;
}



// virtual method
void KCanvasView :: viewportPaintEvent( QPaintEvent* e )
{
QPainter painter;
KPixmap* temp;
TRectangle rect;

  // for reason I don't know it can happen that Qt sends repaint areas with size 0.
  // In these cases we don't have to repaint (actually the following code even crashes
  // for cases with rectangle of size 0)
  if ( e->rect().isValid() == false )  return;

  // find the rect (game coord) which has to be repainted
  // note that we make the rectangle a little bit larger (eps) to avoid rounding (double->int) problems
  // and to allow a minimum size for canvas items (see TCanvasItem::setMinSize)
  // eps is given in pixels
  double eps = (1+fCanvas->getMinSize())/fZoom;
  rect.setRect( e->rect().x()/fZoom-eps , e->rect().y()/fZoom-eps, e->rect().width()/fZoom+2*eps, e->rect().height()/fZoom+2*eps );
  rect.moveBy( contentsX()/fZoom, contentsY()/fZoom );
  temp = new KPixmap( e->rect().size() );
  temp->fill( Qt::black );

  painter.begin( temp );
  painter.translate( -contentsX()-e->rect().x(), -contentsY()-e->rect().y() );

  // Now paint all objects on the pixmap which need repainting
  list<KCanvasItem*>::const_iterator it, it_begin, it_end;
  it_begin = fCanvas->getCanvasItemList()->begin();
  it_end = fCanvas->getCanvasItemList()->end();
  for ( it=it_begin; it!=it_end; it++ )  {
    if ( rect.intersects((*it)->getBoundingBox()) == true )  {
      (*it)->paint( painter, fZoom, rect );
    }
  }

  painter.translate( contentsX()+e->rect().x(), contentsY()+e->rect().y() );
  painter.end();
  bitBlt( viewport(), e->rect().topLeft(), temp, temp->rect() );
  delete temp;
}





//----------------------------------------  private  ----------------------------------------

bool KCanvasView :: validateZoom( double zoom ) const
{
// fixme: check that the size is not going beyond 2*10^9 (int4)
  if ( zoom<0.01 || zoom>100 )  return false;
  return true;
}



