/***************************************************************************
                          kcanvas.h  -  description
                             -------------------
    begin                : Fri Jul 12 2002
    copyright            : (C) 2002 by Marco Krohn
    email                : marco.krohn@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __KCANVAS_INCLUDED
#define __KCANVAS_INCLUDED

#include <list>

#include "kscrollview.h"

#include "common/point.h"
#include "common/rectangle.h"

using std::list;

class KCanvasView;
class KCanvasItem;
class QPainter;
class TRectangle;





//----------------------------------------  class KCanvas  ----------------------------------------

/**
* @class KCanvas
* @brief An enhanced canvas which stores all objects which may need to be shown.
* @author Marco Krohn, <marco.krohn@gmx.de>
* @version $Id: kcanvas.h,v 1.2 2002/07/15 08:48:34 mkrohn5 Exp $
*
* KEnhancedCanvas
*
* Basically stores the list of all items which can be shown to the user.
* All coordinates in this class refer to world coordinates.
* FIXME
* KCanvasItems
* connect KCanvasView
*/
class KCanvas
{
  friend class KCanvasItem;
  friend class KCanvasView;

  public:
    /**
    * Sets the size (world coordinates) of the canvas.
    * @param w Width of the canvas.
    * @param h Height of the canvas.
    */
    KCanvas( double w, double h );
    /**
    * Deletes all canvas items and all views connected to this canvas.
    */
    virtual ~KCanvas();

    void addCanvasItem( KCanvasItem* ci );
    /**
    * Connects another view to this document. Note that
    * this class takes over control of the view, meaning that
    * the view will automatically be deleted.
    * @param view The new view which is connected to this document.
    */
    void addView( KCanvasView* view );
    /**
    * @return The canvas item list @a fCIList.
    */
    const list<KCanvasItem*>* getCanvasItemList() const;
    /**
    * @return Height of canvas.
    */
    double getHeight() const;
    /**
    * @return Width of canvas.
    */
    double getWidth() const;
    /**
    * Removes a view from this document and deletes it.
    * @param view The view which is going to be removed from this canvas.
    * @return @c true if removing the view was successfull, otherwise @c false.
    */
    bool removeView( KCanvasView* view );

  protected:
    /**
    * A list of canvas items.
    */
    list<KCanvasItem*> fCIList;

  private:
    /**
    * Height of the canvas (world coordinates).
    */
    double fHeight;
    /**
    * Width of the canvas (world coordinates).
    */
    double fWidth;
    /**
    * A list of all views connected to this canvas.
    * This list is needed to inform all views if anything has changed.
    */
    list<KCanvasView*> fViewList;
    /**
    * Allows canvas items to have a minimum size.
    * @see KCanvasItem::setMinSize
    */
    int fMinSize;

    /**
    * @return The variable @a fMinSize.
    */
    int getMinSize() const;
    /**
    * Compares the parameter with @a fMinSize and keeps the larger value.
    * @see KCanvasItem::setMinSize
    */
    void setMinSize( int radius );
};





//----------------------------------------  class KCanvasItem  ----------------------------------------

class KCanvasItem
{
  public:
    virtual ~KCanvasItem();

    /**
    * @return Size of the bouding box (world coordinates).
    */
    virtual TRectangle getBoundingBox() const = 0;
    /**
    * Informs all views of this item that this object has changed
    * his state.
    */
    void hasChanged() const;
    /**
    * Paints the item on a paint device using a painter.
    * @param painter The painter for painting on the paint device.
    * @param zoom The current used zoom for viewing the item.
    * @param rect Rectangle (game coordinates) which needs to be repainted.
    *   Note: only under rare circumstances this parameter is useful
    *   for implementing the paint method.
    */
    virtual void paint( QPainter& painter, double zoom, const TRectangle& rect ) const = 0;
    /**
    * This allows you to set a minimum size for the item when it comes to
    * drawing. The general problem is that zooming out very far may actually leave
    * only 1 x 1 pixel for your object. Simply ignoring this and using a larger
    * area with the paint routine leaves you with redraw errors. This method allows
    * you to set a min. size for your object. The drawback is a performace penalty
    * depending very much on your settings (and in general depends quadratic on the
    * @a radius).
    * @param radius The radius of the object (this means you can paint a
    *   (2*radius+1) x (2*radius+1) pixel area without redraw errors).
    */
    void setMinSize( int radius );

  protected:
    KCanvasItem( KCanvas* canvas );

  private:
    /**
    * This link gives us the chance to inform all views very quickly,
    * that something has changed.
    */
    KCanvas* fCanvas;
};





//----------------------------------------  class KCanvasView  ----------------------------------------

class KCanvasView : public KScrollView
{
  Q_OBJECT

  public:
    /**
    * Inits QScrollView and sets @a fZoom to -1 (to mark that this value needs to be set).
    */
    KCanvasView( KCanvas* canvas, QWidget* parent=0, const char* name=0, WFlags w=0 );
    virtual ~KCanvasView();

    /**
    * @return The zoom of this view.
    */
    double getZoom() const;
    /**
    * fixme
    */
    void fastZoomIn( double zoom, int x, int y, int smoothness=1 );
    void fastZoomOut( double zoom, int x, int y, int smoothness=1 );
    /**
    * Repaints the whole viewport. Note: this does call does @e not
    * change @a fCIVList.
    */
    void repaintAll();
    /**
    * Sets the zoom for this view.
    * @param zoom The new value for @a fZoom. @e zoom has to be a value
    *   between 0.01 and 100.
    */
    void setZoom( double zoom );

  signals:
    /**
    * This signal is emitted whenever the zoom changes.
    * The parameter is the new zoom value.
    */
    void zoomChanged( double );
    /**
    * Whenever the position of the mouse pointer changes this signal
    * is emitted (note that zooming can change the mouse position).
    * Here position means the world coordinates.
    */
    void mousePosChanged( TPoint );

  protected:
    /**
    * fixme
    */
    virtual void contentsMouseMoveEvent( QMouseEvent* e );
    /**
    * fixme
    */
    virtual void contentsMousePressEvent( QMouseEvent* e );
    /**
    * This is a low-level painting routine that draws the viewport contents.
    */
    virtual void viewportPaintEvent( QPaintEvent* e );

  private:
    /**
    * The KCanvas object this view is for.
    */
    KCanvas* fCanvas;
    /**
    * The zoom setting for this view.
    */
    double fZoom;

    /**
    * Checks if the zoom is within the limits or not.
    * @param zoom The zoom to validate.
    */
    bool validateZoom( double zoom ) const;
};





#endif        // __KCANVAS_INCLUDED
