#! /usr/bin/perl -w
#
# $Id: act-to-cpp.pl,v 1.6 2002/07/08 06:26:55 mkrohn5 Exp $
#
# .act -> .cpp + .h conversion script
# Written by: Alberto Barsella <ishark@free.fr>
#             Marco Krohn, <marco.krohn@gmx.de>

# Copyright (C) 1999-2002, Alberto Barsella, <ishark@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#

use strict;
use IO::File;


# global variables
my ($action_class, $action_name, $action_type, $action_proto, $action_server);
my $public_private = 'private';


sub switchtoprivate {
    my $hfile = shift;
    
    if ($public_private ne 'private') {
        $hfile->print("\n  private:\n");
        $public_private = 'private';
    }
}


sub switchtopublic {
    my $hfile = shift;
    
    if ($public_private ne 'public') {
        $hfile->print("\n  public:\n");
        $public_private = 'public';
    }
}


sub cleancode {
    my $code = shift;

    # eliminate the empty lines at beginning and end
    while (@$code && $code->[0] =~ /^\s+$/) {
        shift @$code;
    }
    while (@$code && $code->[-1] =~ /^\s+$/) {
        pop @$code;
    }
}



# handlers for the different sections

sub DOincludes {
    my ($hfile, $cfile, $dline, $dfile, $data) = @_;

    print "Defining #include directives....\n";

    # clean up the list of includes
    @$data = map { 
                    if (/^\s*(\S+)\s*$/) {
                        $1;
                    } else {
                        ();
                    }
                } @$data;

    $hfile->print("#line $dline \"$dfile\"\n");
    $cfile->print("#line $dline \"$dfile\"\n");

    map { 
          $hfile->print("#include \"$_\"\n");
          $cfile->print("#include \"$_\"\n");
      } @$data;

    $hfile->print("\n");
    $cfile->print("\n");
}


sub DOserveraction {
    my ($hfile, $cfile, $dline, $dfile, $data) = @_;

    DOaction($hfile, $cfile, $dline, $dfile, $data, 'server');
}



sub DOaction {
    my ($hfile, $cfile, $dline, $dfile, $data, $serverflag) = @_;

    my ($extra_code);

    # conclude any pre-existing action
    DOendaction($hfile, $cfile);

    $action_name = shift @$data;
    $action_name =~ s/^\s*//;
    $action_name =~ s/\s*$//;

    $action_type = uc $action_name;
    $action_type =~ s/ /_/g;
    $action_class = $action_name;
    $action_class =~ s/ //g;
    $action_class = "TAction" . $action_class;

    $action_proto = $action_class . "_proto";

    # is this a server action?
    if ($serverflag && $serverflag eq 'server') {
        $action_server = 1;
        $extra_code = "bool isPlayerAction() const\n      { return false; }\n";
        print "Defining server action '$action_name' ($action_class)....\n";
    } else {
        $action_server = 0;
        $extra_code = '';
        print "Defining action '$action_name' ($action_class)....\n";
    }

    $hfile->print("#line $dline \"$dfile\"\n");
    $cfile->print("#line $dline \"$dfile\"\n");

    # put stuff in .h and .cpp file for declaration
    $hfile->print(<< "EOF");


// Begin declarations for action: $action_name

class $action_class : public TAction {
  private:    
    void initialize();
    void uninitialize();
    virtual bool validate();
    virtual TCanBePrototype* clone() const { return new $action_class(); }

  protected:
    $action_class() : TAction()  { }
    
  public:      
    $action_class(const string& name) : TAction() 
    { 
      declareAsPrototype( name );                                 
      TPMAction::instance().registerPrototype( name, this );
      initialize();    
    }
    ~$action_class() { uninitialize(); }
    
    $extra_code
    virtual bool checkActionConflicts();
    virtual bool resolveActionConflicts();
    virtual bool execute();
    virtual bool load(TDataPackageList* in);
    virtual bool save(TDataPackageList* out) const;
EOF

    $public_private = 'public';

    $cfile->print(<< "EOF");


// Begin definitions for action: $action_name

$action_class* $action_proto = new $action_class(\"$action_type\");

EOF

    # if defining a server action provide the load/save methods here
    if ($action_server) {
        $cfile->print(<< "EOF");


bool ${action_class}::load(TDataPackageList*)
{
  Fatal("$action_class", "load", "server action $action_name does not support load/save", -1);
  return false;
}


bool ${action_class}::save(TDataPackageList*) const
{
  Fatal("$action_class", "save", "server action $action_name does not support load/save", -1);
  return false;
}
EOF
    }   
}



sub DOendaction {
    my ($hfile, $cfile, $dline, $dfile, $data) = @_;
    # data is ignored

    if ($action_name) {
        # pending action, close the classes
        $hfile->print("};\n\n// End declarations for action: $action_name\n\n");
        $cfile->print("\n// End definitions for action: $action_name\n\n");
        undef $action_name;
    }
}



sub DOinternalfields {
    my ($hfile, $cfile, $dline, $dfile, $data) = @_;

    # clean up the list
    @$data = map { 
                    s/^\s+//;
                    s/\s+$//;
                    if ($_) {
                        $_;
                    } else {
                        ();
                    }
                } @$data;

    # anything defined?
    if (@$data) {
        # switch to private
        switchtoprivate($hfile);

        $cfile->print("#line $dline \"$dfile\"\n");

        # output all fields "as is"
        map {
              $hfile->print("    $_;\n");
          } @$data;
    }
}



sub DOfields {
    my ($hfile, $cfile, $dline, $dfile, $data) = @_;
    my (@fields, @f, $save_name, $var_name, $type_name, $parameter_name);

    my $internal_type_match = '^(bool|double|int2|int4|string)$';

    # clean up the list
    @$data = map { 
                    s/^\s+//;
                    s/\s+$//;
                    if ($_) {
                        $_;
                    } else {
                        ();
                    }
                } @$data;

    # anything defined?
    if (@$data) {
        if ($action_server) {
            print "Server action CANNOT have load/save fields (use internal fields)\nabort\n";
            exit;
        }

        # fields are assumed to be private
        switchtoprivate($hfile);

        $hfile->print("#line $dline \"$dfile\"\n");

        # generate both the header and load/save code
        @fields = ();

        map {
              # split, last is "save name", prev is name, rest is type
              @f = split(/ +/, $_);
              $save_name = pop @f;
              $var_name = pop @f;
              $type_name = join(' ', @f);

              # add in header file
              $hfile->print("    $type_name $var_name;\n");

              push @fields, [ $type_name, $var_name, $save_name ];
          } @$data;

        # now we can generate the load code
        # specify parameter name only if it is needed
        $parameter_name = (@fields)? ' in' : '';

        $cfile->print(<< "EOF");


bool ${action_class}::load(TDataPackageList*$parameter_name)
{
  bool bReturn = true;

  bReturn &= TAction::load(in);
EOF

        $cfile->print("#line $dline \"$dfile\"\n");

        # load the fields (type-dependent)
        map {
              if ($_->[0] =~ /$internal_type_match/) {
                $cfile->print("  bReturn &= in->getAndDelete($_->[2], &$_->[1]);\n");
              } else {
                $cfile->print("  bReturn &= $_->[1].load(in, $_->[2], true);\n");
              }
          } @fields;

        # conclude function
        $cfile->print("  return bReturn;\n}\n");

        
        # now we can generate the save code
        # specify parameter name only if it is needed
        $parameter_name = (@fields)? ' out' : '';

        $cfile->print(<< "EOF");


bool ${action_class}::save(TDataPackageList*$parameter_name) const
{
  bool bReturn = true;

  bReturn &= TAction::save(out);
EOF

        $cfile->print("#line $dline \"$dfile\"\n");

        # save the fields (type-dependent)
        map {
              if ($_->[0] =~ /$internal_type_match/) {
                $cfile->print("  bReturn &= out->add($_->[2], &$_->[1]);\n");
              } else {
                $cfile->print("  bReturn &= $_->[1].save(out, $_->[2], true);\n");
              }
          } @fields;

        # conclude function
        $cfile->print("  return bReturn;\n}\n");
    }
}



sub DOinitialize {
    my ($hfile, $cfile, $dline, $dfile, $data) = @_;

    # eliminate the empty lines at beginning and end
    cleancode($data);

    $cfile->print("#line $dline \"$dfile\"\n");

    # write the function, even if empty
    $cfile->print("\n\nvoid ${action_class}::initialize()\n{\n");

    map { $cfile->print($_); } @$data;

    $cfile->print("}\n");
}


sub DOuninitialize {
    my ($hfile, $cfile, $dline, $dfile, $data) = @_;

    # eliminate the empty lines at beginning and end
    cleancode($data);

    $cfile->print("#line $dline \"$dfile\"\n");

    # write the function, even if empty
    $cfile->print("\n\nvoid ${action_class}::uninitialize()\n{\n");

    map { $cfile->print($_); } @$data;

    $cfile->print("}\n");
}


sub DOvalidateobjects {
    my ($hfile, $cfile, $dline, $dfile, $data) = @_;

    # eliminate the empty lines at beginning and end
    cleancode($data);

    # assume a call to TAction::validateObjects
    $cfile->print(<< "EOF");


bool ${action_class}::validate()
{
  if (!TAction::validate()) {
    // if failed, return instantly
    return false;
  }

EOF

    $cfile->print("#line $dline \"$dfile\"\n");

    # output the provided code (even if empty)    
    map { $cfile->print($_); } @$data;

    # set status and return
    $cfile->print("\n  return true;\n}\n");
}



sub DOexecute {
    my ($hfile, $cfile, $dline, $dfile, $data) = @_;

    # eliminate the empty lines at beginning and end
    cleancode($data);

    $cfile->print(<< "EOF");


bool ${action_class}::execute()
{
  if ( validate() == false )
    return false;

EOF
    
    $cfile->print("#line $dline \"$dfile\"\n");

    # output the provided code
    map { $cfile->print($_); } @$data;

    # return success
    $cfile->print("  return true;\n}\n");
}

sub DOcheckconflict {
    my ($hfile, $cfile, $dline, $dfile, $data) = @_;

    # eliminate the empty lines at beginning and end
    cleancode($data);

    $cfile->print(<< "EOF");


bool ${action_class}::checkActionConflicts()
{
  if ( validate() == false )
    return true;

EOF
    
    $cfile->print("#line $dline \"$dfile\"\n");

    # output the provided code
    map { $cfile->print($_); } @$data;

    # return no conflicts
    $cfile->print("  return false;\n}\n");
}


sub DOresolveconflict {
    my ($hfile, $cfile, $dline, $dfile, $data) = @_;

    # eliminate the empty lines at beginning and end
    cleancode($data);

    $cfile->print(<< "EOF");


bool ${action_class}::resolveActionConflicts()
{
  if ( validate() == false )
    return false;

EOF
    $cfile->print("#line $dline \"$dfile\"\n");
    
    # output the provided code
    map { $cfile->print($_); } @$data;

    # return "everything solved"
    $cfile->print("  return true;\n}\n");
}


sub DOmethod {
    my ($hfile, $cfile, $dline, $dfile, $data) = @_;
    my ($method_decl, $method_return, $method_name, $method_params);

    # 1st line is method prototype, split off return type
    $method_decl = shift @$data;
    chomp $method_decl;
    $method_decl =~ s/^\s+//;

    if ($method_decl =~ /\s+([A-Za-z_][A-Za-z_0-9]+)\s*\(/) {
        $method_return = $`;
        $method_name = $1;
        $method_params = "($'";

        # add to header file, as public
        switchtopublic($hfile);
        $hfile->print("#line $dline \"$dfile\"\n");
        $hfile->print("    $method_return $method_name$method_params;\n");

        # add the code
        # eliminate the empty lines at beginning and end
        cleancode($data);

        $cfile->print("#line $dline \"$dfile\"\n");
        $cfile->print("\n\n$method_return ${action_class}::$method_name$method_params\n{\n");
        # output the provided code
        map { $cfile->print($_); } @$data;

        $cfile->print("}\n");
        
    } else {
        print "Unable to parse method declaration: $method_decl....\naborted";
        exit;
    }
}


my %sections = (
     "INCLUDES"         => \&DOincludes,
     "ACTION"           => \&DOaction,
     "SERVER_ACTION"    => \&DOserveraction,
     "INTERNAL_FIELDS"  => \&DOinternalfields,
     "FIELDS"           => \&DOfields,
     "INITIALIZE"       => \&DOinitialize,
     "UNINITIALIZE"     => \&DOuninitialize,
     "VALIDATE_OBJECTS" => \&DOvalidateobjects,
     "EXECUTE"          => \&DOexecute,
     "CHECK_CONFLICT"   => \&DOcheckconflict,
     "RESOLVE_CONFLICT" => \&DOresolveconflict,
     "METHOD"           => \&DOmethod
);


# ------- program starts

my $input_file;

while (@ARGV) {
    $input_file = shift;

    if ($input_file =~ /\.act$/) {
        # file type is ok
        if (-e $input_file) {
            # process the file
            my $output_header = $input_file;
            my $output_cpp = $input_file;
            $output_header =~ s/\.act$/\.h/;
            $output_cpp =~ s/\.act$/\.cpp/;

            process_act_file($input_file, $output_header, $output_cpp);

        } else {
            print STDERR "Input file '$input_file' not found, skipping....\n";
        }
    } else {
        print STDERR "Input file '$input_file' does not end in .act, skipping....\n";
    }
}

exit(0);


sub process_act_file {
    my ($infspec, $outh, $outcpp) = @_;
    my ($l, $headerguard, $curr_section, @curr_data, $s, $new_section,
        $section_startline);
    my ($infile, $ohfile, $ocfile);

    print "*********** $infspec -> $outcpp + $outh.....\n";

    # will comment out when everything is ok
#    if (-e $outcpp || -e $outh) {
#        print STDERR "Output files already exist, I do nothing....\n";
#        return;
#    }

    $infile = new IO::File($infspec);
    defined $infile or die "Failed to open '$infspec': $!\n";

    $ohfile = new IO::File(">$outh");
    defined $ohfile or die "Can't write to '$outh': $!\n";

    $ocfile = new IO::File(">$outcpp");
    defined $ocfile or die "Can't write to '$outcpp': $!\n";

    # first, initialize the files
    $ohfile->print(<< "EOF");
//
// THIS FILE IS AUTOMATICALLY GENERATED
// ----------- DO NOT EDIT!!! ---------
// All modifications will be lost!!!!!!
//
// File generated from $infspec
//

EOF

    $ocfile->print(<< "EOF");
//
// THIS FILE IS AUTOMATICALLY GENERATED
// ----------- DO NOT EDIT!!! ---------
// All modifications will be lost!!!!!!
//
// File generated from $infspec
//

EOF

    # put an include guard in the header file
    $headerguard = $outh;
    $headerguard =~ s/\.h//;
    $headerguard = "__" . uc($headerguard) . "_INCLUDED";

    $ohfile->print( << "EOF");
#ifndef $headerguard
#define $headerguard

EOF

    $ohfile->print("#include \"gamelogic/action.h\"\n");
    $ocfile->print("#include \"gamelogic/$outh\"\n\n");

    undef $curr_section;
    @curr_data = ();

    while ( ($l = $infile->getline()) ) {
        # ignore comments
        next
            if ($l =~ /^#.*/);

        # check if we have a section change
        $new_section = 0;
        foreach $s (keys %sections) {
            if ($l =~ /^$s:/) {
                # found a new section, terminate any previous one
                &{$sections{$curr_section}}($ohfile, $ocfile,
                                            $section_startline, $infspec,
                                            \@curr_data)
                    if ($curr_section);

                $new_section = 1;
                $section_startline = $.;
                $curr_section = $s;
                @curr_data = ($');

                last;
            }
        }

        push @curr_data, $l
            if (!$new_section);
    }

    &{$sections{$curr_section}}($ohfile, $ocfile,
                                $section_startline, $infspec,
                                \@curr_data)
        if ($curr_section);

    DOendaction($ohfile, $ocfile);

    # close the include guard
    $ohfile->print("\n#endif\n");

    $infile->close();
    $ohfile->close();
    $ocfile->close();
}
