#ifndef _LIST_H_
#define _LIST_H_

// ----------------------------------------------------------------------------
//
// User friendly generic double linked lists
//
// Copyright (C) 1996-1999, Pier Donini, Pier.Donini@epfl.ch
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 1, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details. 
//
// Comments:
//
//   A TList has the following public methods:
//
//     TList()
//       Creates an empty list.
//
//     ~TList() 
//       Removes the elements from the list, and destroys it. It does *not*
//       destroy the elements that might have been newed (user responsability).
//  
//     uint Size()		   		Number of elements in the list
//     bool IsEmpty()				TRUE if the list is empty
//  
//     void Reset()				Go to the beginning of the list
//     void ToTail()				Go to the end of the list
//     bool AtTail()				TRUE if at the end
//     bool AtHead()				TRUE if at the start 
//
//     T Next()
//     T Previous()
//     T& Current()
//     T& operator[](uint Index)
// 
//     void AddFirst(T Data)
//     void AddLast(T Data)
//     void Add(uint Index, T Data)
//     void Insert(T Data)			Inserts before current element
//  
//     T DetachCurrent(void DoAction(T)) 	 DoAction is one of the DoXxx:
//     T Detach(uint Index, void DoAction(T))   - DoNone
//     void FlushToTail(void DoAction(T))        - DoFree
//     void FlushToHead(void DoAction(T))        - DoFreeArray
//     void Flush(void DoAction(T));
//
//       Use 'DoNone' when the element was not newed or you don't want to
//       delete it yet, use 'DoFree' to delete the elements (that must then
//       have been newed earlier) and use 'DoFreeArray' for dynamically
//       allocated arrays.
//
//     void Sort(int Greater(T*, T*))
//       If '>' and '<' are defined for type T, use 'BasicGreater', otherwise
//       compute one yourselves (see BasicGreater below).
//
//     void Append(TList<T> *List)		Warning! Destroys *List.
//
//     void ForEach(void fn(T &, void *), void *Data = NULL);
//       Applies the function fn to each element of the list with the Data
//       parameter (void *).
//
// ----------------------------------------------------------------------------


#include <assert.h>


//
// List Element
//

template <class T> class TListElement
{
public:
  T fData;
  TListElement *fNext;
  TListElement *fPrevious;

  TListElement();
  TListElement(T Data);
  TListElement(T Data, TListElement *Next, TListElement *Previous);
};


template <class T> TListElement<T>::TListElement()
{
  fNext = NULL;
  fPrevious = NULL;
  // Assumes T has a default constructor
};


template <class T> TListElement<T>::TListElement(T Data)
{
  fData = Data;
};


template <class T> TListElement<T>::TListElement
(T Data, TListElement *Next, TListElement *Previous)
{
  fData = Data;
  fNext = Next;
  fPrevious = Previous;
};


//
// Templates functions used with TLists methods parameters
//

template <class T> void DoNone(T ) { };
template <class T> void DoFree(T Data) { delete Data; }
template <class T> void DoFreeArray(T Data) { delete[] Data; }

template <class T> int BasicGreater(T *One, T *Two) 
{ 
  if (*One > *Two) return 1;
  if (*One < *Two) return -1;
  return 0;
}


//
// List
//

template <class T> class TList
{
private:
  unsigned int fSize;
  TListElement<T> *fCurrent;
  TListElement<T> *fHead;
  TListElement<T> *fTail;
  void DetachElement(TListElement<T> *El);
  void AttachElement(TListElement<T> *El, TListElement<T> *Before);
  void SortFromTo(int Greater(T*, T*), TListElement<T> *, TListElement<T> *);
  
public:
  TList();
  ~TList();
  
  unsigned int Size();
  bool IsEmpty();
  
  void Reset();
  void ToTail();
  bool AtTail();
  bool AtHead();
  T Next();
  T Previous();
  T& Current();
  T& operator[](unsigned int Index);
 
  void AddFirst(T Data);
  void AddLast(T Data);
  void Add(unsigned int Index, T Data);
  void Insert(T Data);
  
  T DetachCurrent(void DoAction(T)); 		// DoAction is one of the DoXxx
  T Detach(unsigned int Index, void DoAction(T));
  void FlushToTail(void DoAction(T));
  void FlushToHead(void DoAction(T));
  void Flush(void DoAction(T));

  void Sort(int Greater(T*, T*)); 		// Greater can be BasicGreater
  void Append(TList<T> *List);
  void ForEach(void fn(T &, void *), void *Data = NULL);
};


template <class T> TList<T>::TList()
{
  fHead = new TListElement<T>;
  fTail = new TListElement<T>;
  fCurrent = fHead;
  fTail->fPrevious = fHead;
  fHead->fNext = fTail;
  fSize = 0;
}


template <class T> TList<T>::~TList()
{
  Flush(DoNone);
  delete fHead;
  delete fTail;
}


template <class T> void TList<T>::DetachElement(TListElement<T> *El)
{
  fSize--;
  El->fPrevious->fNext = El->fNext;
  El->fNext->fPrevious = El->fPrevious;
}


template <class T> void TList<T>::AttachElement
(TListElement<T> *El, TListElement<T> *Before)
{
  El->fNext = Before;
  El->fPrevious = Before->fPrevious;
  Before->fPrevious->fNext = El;
  Before->fPrevious = El;
  fSize++;
}


template <class T> unsigned int TList<T>::Size()
{
  return fSize;
}


template <class T> bool TList<T>::IsEmpty()
{
  return fSize == 0;
}


template <class T> void TList<T>::Reset()
{
  fCurrent = fHead;
}


template <class T> void TList<T>::ToTail()
{
  fCurrent = fTail;
}


template <class T> bool TList<T>::AtTail()
{
  return fCurrent->fNext == fTail;
}


template <class T> bool TList<T>::AtHead()
{
  return fCurrent->fPrevious == fHead;
}


template <class T> T TList<T>::Next()
{
  assert(fCurrent != fTail && fCurrent->fNext != fTail);

  return (fCurrent = fCurrent->fNext)->fData;
}


template <class T> T TList<T>::Previous()
{
  assert(fCurrent != fHead && fCurrent->fPrevious != fHead);

  return (fCurrent = fCurrent->fPrevious)->fData;
}


template <class T> T& TList<T>::Current()
{
  assert(fCurrent != fHead && fCurrent != fTail);

  return fCurrent->fData;
}


template <class T> T& TList<T>::operator[](unsigned int Index)
{
  TListElement<T> *cur;
  unsigned int i;
  
  assert(Index < fSize);
  
  for (cur = fHead->fNext, i = 0; i < Index; cur = cur->fNext, i++);
  
  return cur->fData;
}


template <class T> void TList<T>::AddFirst(T Data)
{
  TListElement<T> *el;

  el = new TListElement<T>(Data, fHead->fNext, fHead);
  fHead->fNext->fPrevious = el;
  fHead->fNext = el;
  fSize++;
}


template <class T> void TList<T>::AddLast(T Data)
{
  AttachElement(new TListElement<T>(Data), fTail);
}


template <class T> void TList<T>::Add(unsigned int Index, T Data)
{
  unsigned int i;
  TListElement<T> *cur;
  
  assert(Index <= fSize);
  
  for (cur = fHead->fNext, i = 0; i < Index; cur = cur->fNext, i++);

  AttachElement(new TListElement<T>(Data), cur);
}


template <class T> void TList<T>::Insert(T Data)
{
  assert(fCurrent != fHead);
  
  AttachElement(new TListElement<T>(Data), fCurrent);
}


template <class T> T TList<T>::DetachCurrent(void DoAction(T))
{
  TListElement<T> *cur = fCurrent;
  T data = cur->fData;

  assert(fCurrent != fHead && fCurrent != fTail);

  fCurrent = fCurrent->fPrevious;
  DetachElement(cur);
  DoAction(cur->fData);
  delete cur;
  
  return data;
}


template <class T> T TList<T>::Detach(unsigned int Index, void DoAction(T))
{
  unsigned int i;
  TListElement<T> *cur;
  T data;
  
  assert(Index < fSize);
  
  for (cur = fHead->fNext, i = 0; i < Index; cur = cur->fNext, i++);
  if (fCurrent == cur)
    fCurrent = fCurrent->fPrevious;
  data = cur->fData;
  DetachElement(cur);
  DoAction(cur->fData);
  delete cur;

  return data;
}


template <class T> void TList<T>::FlushToTail(void DoAction(T data))
{
  TListElement<T> *cur, *prev;
  
  assert(fCurrent != fTail);

  for (cur = fTail->fPrevious; cur != fCurrent; cur = prev)
  {
    prev = cur->fPrevious;
    DoAction(cur->fData);
    delete cur;
    fSize--;
  }
  fCurrent->fNext = fTail;
  fTail->fPrevious = fCurrent;
}


template <class T> void TList<T>::FlushToHead(void DoAction(T data))
{
  TListElement<T> *cur, *next;
  
  assert(fCurrent != fTail);

  for (cur = fHead->fNext; cur != fCurrent; cur = next)
  {
    next = cur->fNext;
    DoAction(cur->fData);
    delete cur;
    fSize--;
  }
  fCurrent->fPrevious = fHead;
  fHead->fNext = fCurrent;
}


template <class T> void TList<T>::Flush(void DoAction(T data))
{
  fCurrent = fHead;
  FlushToTail(DoAction);
}


template <class T> void TList<T>::Append(TList<T> *List)
/*
   Destroys List. A very bad idea would be to call List.Append(&List)...
*/
{
  assert(List != this);
  
  if (List->fSize)
  {
    fTail->fPrevious->fNext = List->fHead->fNext;
    List->fTail->fPrevious->fNext = fTail;
    List->fHead->fNext->fPrevious = fTail->fPrevious;
    fTail->fPrevious = List->fTail->fPrevious;
    
    List->fTail->fPrevious = List->fHead;
    List->fHead->fNext = List->fTail;
    
    fSize += List->fSize;
    List->fSize = 0;
  }
}


template <class T> void TList<T>::SortFromTo
(int Greater(T *One, T *Two), TListElement<T> *Head, TListElement<T> *Tail)
// Requires Head->Next != Tail->Previous (i.e. Sublist size > 1)
{
  TListElement<T> *Middle, *Current, *Next, *Last;
  
  for (Current = Head->fNext, Middle = Tail->fPrevious;
       Current != Middle && Current->fNext != Middle;
       Current = Current->fNext, Middle = Middle->fPrevious);
  
  Last = Tail->fPrevious;
  
  for (Current = Head->fNext; Current != Middle; Current = Next)
  {
    Next = Current->fNext;
    if (Greater(&(Current->fData), &(Middle->fData)) > 0)
    {
      DetachElement(Current);
      AttachElement(Current, Tail);
    }
  }
  if (Middle != Last)
  {
    Last = Last->fNext;
    for (Current = Middle->fNext; Current != Last; Current = Next)
    {
      Next = Current->fNext;
      if (Greater(&(Current->fData), &(Middle->fData)) < 0)
      {
        DetachElement(Current);
        AttachElement(Current, Middle);
      }
    }
  }
  // Sublists' size must be not 0 nor 1
  if (Head->fNext != Middle && Head->fNext != Middle->fPrevious)
    SortFromTo(Greater, Head, Middle);
  if (Middle->fNext != Tail && Middle->fNext != Tail->fPrevious)
    SortFromTo(Greater, Middle, Tail);
}


template <class T> void TList<T>::Sort(int Greater(T *One, T *Two))
{
  if (fSize > 1)
    SortFromTo(Greater, fHead, fTail);
}


template <class T> void TList<T>::ForEach(void fn(T &, void *), void *Data)
{
  for (TListElement<T> *cur = fHead->fNext; cur != fTail ; cur = cur->fNext)
    fn(cur->fData, Data);
}


#endif // __LIST_H
