/***************************************************************************
                          log2html.h  -  description
                             -------------------
    begin                : Sat Apr 8 2000
    copyright            : (C) 2000 by Marco Krohn
    email                : mkrohn@stud.uni-hannover.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __CONVERT_INCLUDED
#define __CONVERT_INCLUDED

#include <fstream>
#include <list>
#include <string>

#include "stringtools.h"

using namespace std;



/**
* TMessage is a class which stores all information of a message,
* that means: the message itself, the indentation, the function and
* the class which sent the message.
*
* @short Stores all needed information of a message.
*/
class TMessage
{
  public:
    enum
    {
      MESSAGE,
      WARNING,
      ERROR
    };

    /**
    * The constructor initializes the object with standard values,
    * e.g. class name = "unknown".
    */
    TMessage();

    /**
    * Returns the name of the class which sent this message.
    * @returns The class name.
    */
    string GetClassName() const      { return sClassName; }
    /**
    * Returns the name of the function which sent this message.
    * @returns The function name.
    */
    string GetFunctionName() const   { return sFunctionName; }
    int GetIndentation() const       { return iIndent; }
    string GetMessage() const        { return sMessage; }
    bool GetReadStatus() const       { return bReadStatus; }
    int GetType() const              { return iType; }
    /**
    * The function reads a message from the input stream. It reads the message itself,
    * the class and function name, the priority and the indentation.
    * @param A reference to the input stream.
    * @returns true=message read correctly, false=end of file reached (no message read)
    */
    bool ReadMessage(ifstream& in);
    void SetIndentation(int iIndentation) { iIndent=iIndentation; }
    void SetMessage(const string& s) { sMessage=s; }

  private:
    int iIndent, iType;
    string sClassName, sFunctionName, sMessage;
    bool bQouteMode, bReadStatus;

    void ParseComment(string& sLine);
    bool ParseLine(string& sLine);
};





class TErrorFile
{
  public:
    TErrorFile(const string& sFileName);
    ~TErrorFile();

    void HandleMessage(TMessage* pMessage, int iInfoCounter);

  private:
    ofstream out;

    void BeginHTML();
    void EndHTML();
};





/**
*
* @short This class handles a set of messages.
*/
class TLayer
{
  public:
    /**
    * The constructor simply stores the information within the class,
    * opens the output file and calls BeginHTML().
    * @param sBaseFileName The file name of the hmtl file without the ".html" extension.
    * @param iMin The lowest indent number which should be handled by this layer.
    * @param iMax The highest indent number which should be handled by this layer.
    * @param pParentLayer A pointer to the parent layer.
    */
    TLayer(const string& sBaseFileName, int iMin, int iMax, const TLayer* pParentLayer=0);
    /**
    * Deletes the child layer, writes the end of the html file and closes the file.
    */
    ~TLayer();

    /**
    * The function handles a message, 2 cases are possible:
    * the message can be handled by the layer itself,
    * the message has a higher priority number which means that a child layer has
    * to handle the message.
    * @param pMessage A pointer to the message which must be handled. The class TLayer
    *   is responsible for deleting the message if it is not used anymore.
    * @param pNext A pointer to the next message (after pMessage). This information is
    *   only needed to set a link if the next message has a lower priority.
    */
    void HandleMessage(TMessage* pMessage, const TMessage* pNext=0);

  private:
    /**
    * All output will be send to this output stream.
    */
    ofstream out;

    /**
    * A pointer to the last message this layer has written to the output stream.
    */
    TMessage *pMessage;
    /**
    * The base file name without ".html" extension, e.g. "sl-1-4-3".
    */
    string sBaseFileName;
    /**
    * Pointer to a child layer if a child layer does exist. Otherwise this pointer
    * is set to 0.
    */
    TLayer* pChildLayer;
    /**
    * A pointer to the parent layer which is for example needed to write the
    * history to the output stream. If the parent layer pointer is set to 0, then
    + this is the top layer.
    */
    const TLayer* pParentLayer;
    /**
    *  The range of priority numbers which is handled by this layer.
    */
    int iMin, iMax;
    /**
    * The number of the next child layer which is used to determine the
    * correct file name, e.g. if the base file name of this layer is
    * "sl-1-4-3" and iLayerCounter=5, then the base file name of the next
    * child layer generated by this layer will be "sl-1-4-3-5".
    */
    int iLayerCounter;
    static int iInfoCounter;

    /**
    * The standard stuff for all html files.
    */
    void BeginHTML();
    void CreateInfoFile();
    void CreateLinkMessage();
    void CreateNewLayer();
    void EndHTML();
    string MakeChildName();
    void MakeLayerUpButton();
    void WriteHistory();
    void WriteMessage(string sChildFileName="");
};





/**
*
* @short Reads the messages in a list and handle these messages.
*/
class TConvert
{
  public:
    /**
    * Simply sets the EOF flag to false.
    */
    TConvert();
    /**
    *
    * @param sFileName The name of the file which should be converted.
    *        In almost all cases the file name is "log.dat".
    */
    void ConvertFile(const string& sFileName);

  private:
    /**
    * The input stream.
    */
    ifstream in;
    /**
    * A flag which stores the information if the end of file has been reached.
    * (true means end of file reached)
    */
    bool bEOF;
    /**
    * This list stores always some pointers to messages which are not handled.
    * If the list is empty then all messages were handled.
    */
    list<TMessage*> MessageList;

    /**
    * Creates the main html file, called "index.html".
    * @param sFileName The name of the project, e.g. "sl".
    */
    void CreateMainHTML(const string& sFileName);
    /**
    * If there are less than 5 messages in the list this function
    * tries to add 10 more messages to the message list. If the end of
    * file is reached no more messages are added.
    */
    void FillMessageList();
};





#endif // __CONVERT_INCLUDED